package com.krispot.client.ui;

import com.google.gwt.dom.client.Style;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.krispot.client.common.CSSStyle;
import com.krispot.client.common.Reset;
import com.krispot.client.common.Validate;
	
public class KriPasswordMeter  implements KeyUpHandler, Reset, Validate {
	private PasswordTextBox oldPwdTextBox = new PasswordTextBox();
	private PasswordTextBox passwordTextBox = new PasswordTextBox();
	private PasswordTextBox confirmPasswordTextBox = new PasswordTextBox();
	private VerticalPanel verticalPanel = new VerticalPanel(); 
	private HTML message = new HTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font>"); 
	private HTML confirmPasswordMessage = new HTML("&nbsp;"); 
	private HorizontalPanel bar = new HorizontalPanel();
	private HorizontalPanel colorBar = new HorizontalPanel();
	private Style colorBarStyle = colorBar.getElement().getStyle();
	private KriShowMessage errorMessageOn;
	private String errorMessage;
	public KriPasswordMeter(String errorMessage, KriShowMessage errorMessageOn) {
		this();
		this.errorMessageOn = errorMessageOn;
		this.errorMessage = errorMessage;
	}
	
	public KriPasswordMeter() {
		
		colorBar.setHeight("8px");
		colorBar.add(new HorizontalPanel());
		bar.setSize("150px", "8px");
		bar.setBorderWidth(0);
		bar.getElement().getStyle().setBackgroundColor("#F0F8FF");
		bar.add(colorBar);
		passwordTextBox.addKeyUpHandler(this);
		confirmPasswordTextBox.addKeyUpHandler(this);
		verticalPanel.setSize("100%", "30px");
		verticalPanel.setBorderWidth(0);
		verticalPanel.add(message);
		verticalPanel.setCellHorizontalAlignment(message, HasHorizontalAlignment.ALIGN_LEFT);
		verticalPanel.setCellVerticalAlignment(message, HasVerticalAlignment.ALIGN_MIDDLE);
		verticalPanel.add(bar);
		verticalPanel.setCellHorizontalAlignment(bar, HasHorizontalAlignment.ALIGN_LEFT);
		verticalPanel.setCellVerticalAlignment(bar, HasVerticalAlignment.ALIGN_MIDDLE);
		
	}

	public Widget getConfirmPasswordTextBox() {
		HorizontalPanel horizontalPanel = new HorizontalPanel();
		CSSStyle.maskedTextBox(horizontalPanel);
		CSSStyle.zeroBorder(confirmPasswordTextBox);
		horizontalPanel.add(confirmPasswordTextBox);
		horizontalPanel.setCellVerticalAlignment(confirmPasswordTextBox, HasVerticalAlignment.ALIGN_MIDDLE);
		horizontalPanel.setCellWidth(confirmPasswordTextBox, "170px");
		horizontalPanel.setBorderWidth(0);
		horizontalPanel.setSpacing(0);
		horizontalPanel.setHeight("28px");
		return horizontalPanel;
	}
	
	public String getOldPwd() {
		return oldPwdTextBox.getText();
	}
	public Widget getOldPwdTextBox() {
		HorizontalPanel horizontalPanel = new HorizontalPanel();
		CSSStyle.maskedTextBox(horizontalPanel);
		CSSStyle.zeroBorder(oldPwdTextBox);
		horizontalPanel.add(oldPwdTextBox);
		horizontalPanel.setCellWidth(oldPwdTextBox, "170px");
		horizontalPanel.setBorderWidth(0);
		horizontalPanel.setSpacing(0);
		horizontalPanel.setHeight("28px");
		return horizontalPanel;
	}

	public HTML getConfirmPasswordMisMatch() {
		return confirmPasswordMessage;
	}
	
	public void reset() {
		oldPwdTextBox.setText("");
		passwordTextBox.setText("");
		confirmPasswordTextBox.setText("");
	}
	
	public Widget getPasswordTextBox() {
		HorizontalPanel horizontalPanel = new HorizontalPanel();
		CSSStyle.maskedTextBox(horizontalPanel);
		CSSStyle.zeroBorder(passwordTextBox);
		horizontalPanel.add(passwordTextBox);
		horizontalPanel.setCellVerticalAlignment(passwordTextBox, HasVerticalAlignment.ALIGN_MIDDLE);
		horizontalPanel.setCellWidth(passwordTextBox, "170px");
		horizontalPanel.setBorderWidth(0);
		horizontalPanel.setSpacing(0);
		horizontalPanel.setHeight("28px");
		return horizontalPanel;
	}
	
	public String getNewPwd() {
		return passwordTextBox.getText();
	}
	
	public Widget getPasswordMeter() {
		return verticalPanel;
	}
	
    private static final int PASSWORD_LEVEL_VERYWEAK = 0; 
    private static final int PASSWORD_LEVEL_WEAK = 1; 
    private static final int PASSWORD_LEVEL_MEDIOCRE = 2; 
    private static final int PASSWORD_LEVEL_STRONG = 3; 
    private static final int PASSWORD_LEVEL_VERYSTRONG = 4; 
	 public  int getPasswordStrength(String passwd) { 
         boolean upper = false, 
         lower = false, 
         numbers = false, 
         special = false; 
         int intScore = 0, length = 0; 


         if (passwd == null) 
                 return PASSWORD_LEVEL_VERYWEAK; 
         // PASSWORD LENGTH 
         length = passwd.length(); 
         if (length < 5) { // length 4 or less 
             intScore = (intScore + 3); 
         } else if (length > 4 && passwd.length() < 8) {// length between 5 and 7 
             intScore = (intScore + 6); 
         } else if (length > 7 && passwd.length() < 16) { // length between 8 and 15 
                 intScore = (intScore + 12); 
         } else if (length > 15) {// length 16 or more 
             intScore = (intScore + 18); 
         } 
         // LETTERS 

         if (passwd.matches(".*[a-z]+.*")) 
         { 
             intScore = (intScore + 1); 
             lower=true; 
         } 

         if (passwd.matches(".*[A-Z]+.*")) 
         { 
             intScore = (intScore + 5); 
             upper=true; 
         } 


         // NUMBERS 
         if (passwd.matches(".*[0-9]+.*")) 
         { 
             intScore = (intScore + 5); 
             numbers=true; 
         } 

         if (passwd.matches(".*[0-9].*[0-9].*[0-9].*")) 
             intScore = (intScore + 3); 

         // SPECIAL CHAR 
         if (passwd.matches(".*[:,!,@,#,$,%,^,&,*,?,_,~]+.*")) 
         { 
             intScore = (intScore + 5); 
             special=true; 
         } 


         if (passwd.matches(".*[:,!,@,#,$,%,^,&,*,?,_,~].*[:,!,@,#,$,%,^,&,*,?,_,~].*")) 
             intScore = (intScore + 3); 

         // COMBOS 
         if (upper && lower ) // [verified] both upper and lower case 
         { 
             intScore = (intScore + 2); 
         } 
         if ((upper || lower ) && numbers ) // [verified] both letters and numbers 
         { 
         	intScore = (intScore + 2); 
         } 
         if ((upper || lower ) && numbers && special) // [verified] letters, numbers, and special characters 
         { 
             intScore = (intScore + 2); 
         } 
         if (upper  && lower  && numbers  && special ) // [verified] upper, lower, numbers, and special characters 
         { 
             intScore = (intScore + 2); 
         } 
         //System.out.println("Mdp: "+passwd+" Score :" +intScore); 
         if (intScore < 16) 
                 return PASSWORD_LEVEL_VERYWEAK; 
         else if (intScore > 15 && intScore < 25) 
                 return PASSWORD_LEVEL_WEAK; 
         else if (intScore > 24 && intScore < 35) 
                 return PASSWORD_LEVEL_MEDIOCRE; 
         else if (intScore > 34 && intScore < 45) 
                 return PASSWORD_LEVEL_STRONG; 
         else 
                 return PASSWORD_LEVEL_VERYSTRONG; 
 } 
	 public static int CheckPasswordStrength(String passwd) { 
         boolean upper = false, 
         lower = false, 
         numbers = false, 
         special = false; 
         int intScore = 0, length = 0; 


         if (passwd == null) 
                 return PASSWORD_LEVEL_VERYWEAK; 
         // PASSWORD LENGTH 
         length = passwd.length(); 
         if (length < 5) { // length 4 or less 
             intScore = (intScore + 3); 
         } else if (length > 4 && passwd.length() < 8) {// length between 5 and 7 
             intScore = (intScore + 6); 
         } else if (length > 7 && passwd.length() < 16) { // length between 8 and 15 
                 intScore = (intScore + 12); 
         } else if (length > 15) {// length 16 or more 
             intScore = (intScore + 18); 
         } 
         // LETTERS 

         if (passwd.matches(".*[a-z]+.*")) 
         { 
             intScore = (intScore + 1); 
             lower=true; 
         } 

         if (passwd.matches(".*[A-Z]+.*")) 
         { 
             intScore = (intScore + 5); 
             upper=true; 
         } 


         // NUMBERS 
         if (passwd.matches(".*[0-9]+.*")) 
         { 
             intScore = (intScore + 5); 
             numbers=true; 
         } 

         if (passwd.matches(".*[0-9].*[0-9].*[0-9].*")) 
             intScore = (intScore + 3); 

         // SPECIAL CHAR 
         if (passwd.matches(".*[:,!,@,#,$,%,^,&,*,?,_,~]+.*")) 
         { 
             intScore = (intScore + 5); 
             special=true; 
         } 


         if (passwd.matches(".*[:,!,@,#,$,%,^,&,*,?,_,~].*[:,!,@,#,$,%,^,&,*,?,_,~].*")) 
             intScore = (intScore + 3); 

         // COMBOS 
         if (upper && lower ) // [verified] both upper and lower case 
         { 
             intScore = (intScore + 2); 
         } 
         if ((upper || lower ) && numbers ) // [verified] both letters and numbers 
         { 
         	intScore = (intScore + 2); 
         } 
         if ((upper || lower ) && numbers && special) // [verified] letters, numbers, and special characters 
         { 
             intScore = (intScore + 2); 
         } 
         if (upper  && lower  && numbers  && special ) // [verified] upper, lower, numbers, and special characters 
         { 
             intScore = (intScore + 2); 
         } 
         //System.out.println("Mdp: "+passwd+" Score :" +intScore); 
         if (intScore < 16) 
                 return PASSWORD_LEVEL_VERYWEAK; 
         else if (intScore > 15 && intScore < 25) 
                 return PASSWORD_LEVEL_WEAK; 
         else if (intScore > 24 && intScore < 35) 
                 return PASSWORD_LEVEL_MEDIOCRE; 
         else if (intScore > 34 && intScore < 45) 
                 return PASSWORD_LEVEL_STRONG; 
         else 
                 return PASSWORD_LEVEL_VERYSTRONG; 
	 }
	@Override
	public void onKeyUp(KeyUpEvent event) {
		
		String text = ((TextBox)event.getSource()).getText();
		
		if(event.getSource() == confirmPasswordTextBox){
			
			if(passwordTextBox.getText().equals(confirmPasswordTextBox.getText())){
				confirmPasswordMessage.setHTML("<font size='1px' color='GREEN'><b>Password Match</b></font>");
			}else{
				confirmPasswordMessage.setHTML("<font size='1px' color='RED'><b>Password Miss Match</b></font>");
			}
			if(confirmPasswordTextBox.getText().length()==0) {
				confirmPasswordMessage.setHTML("&nbsp;");
			}
			return;
		}
		
		if(text.length() == 0){
			message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font>");
			colorBar.setWidth("0%");
			return;
		}else if(text.length() > 0 && text.length() < 6){
			message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:<b>&nbsp;&nbsp;Too Short</b></font>");
			colorBar.setWidth("0%");
			return;
		}
		    switch ( getPasswordStrength(text) ) {
			case PASSWORD_LEVEL_VERYWEAK:
				message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font><font size='1px' color='BROWN'><b>&nbsp;&nbsp;Very Weak</b></font>");				
				colorBar.setWidth("20%");
				colorBarStyle.setBackgroundColor("BROWN");
				break;
			case PASSWORD_LEVEL_WEAK:
				message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font><font size='1px' color='#DAA520'><b>&nbsp;&nbsp;Week</b></font>");
				colorBar.setWidth("40%");
				colorBarStyle.setBackgroundColor("#DAA520");
				break;
			case PASSWORD_LEVEL_MEDIOCRE:
				message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font><font size='1px' color='#FF8C00'><b>&nbsp;&nbsp;Medium</b></font>");
				colorBar.setWidth("60%");
				colorBarStyle.setBackgroundColor("#FF8C00");
				break;
			case PASSWORD_LEVEL_STRONG:
				message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font><font size='1px' color='#8B008B'><b>&nbsp;&nbsp;Strong</b></font>");
				colorBar.setWidth("80%");
				colorBarStyle.setBackgroundColor("#8B008B");
				break;
			case PASSWORD_LEVEL_VERYSTRONG:
				message.setHTML("<font size='1px' color='blue' weight='BOLD'>Password strength:</font><font size='1px' color='GREEN'><b>&nbsp;&nbsp;Very Strong</b></font>");
				colorBar.setWidth("100%");
				colorBarStyle.setBackgroundColor("GREEN");
				break;
			}
	}

	@Override
	public boolean isValidate() {
		String pwd = "" + passwordTextBox.getText();
		String cfmPwd = "" + confirmPasswordTextBox.getText();
		if(pwd.trim().equals(cfmPwd.trim())) {
			return true;
		}
		errorMessageOn.setError(errorMessage);
		return false;
	} 
}
