package com.krispot.client.ui;

import com.gargoylesoftware.htmlunit.protocol.about.Handler;
import com.google.gwt.dom.client.Style;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.dom.client.Style.VerticalAlign;
import com.google.gwt.event.dom.client.BlurEvent;
import com.google.gwt.event.dom.client.BlurHandler;
import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.TextBox;
import com.krispot.client.common.CSSStyle;
import com.krispot.client.common.Reset;
import com.krispot.client.common.Validate;
import com.krispot.client.common.Value;
	
public class KriTextBox extends HorizontalPanel implements KeyUpHandler, Validate, Reset, Value {
	
	private TextBox textBox = new TextBox();
	private Image image = new Image("images/correct16.png");
	private String regExp;
	private KriShowMessage errorMessageOn;
	private String errorMessage;
	
	public void setEnabled(boolean enabled) {
		textBox.setEnabled(enabled);
	}
	public void addBlurHandler(BlurHandler handler){
		textBox.addBlurHandler(handler);
	}
	public void addFocusHandler(FocusHandler handler) {
		textBox.addFocusHandler(handler);
	}
	public TextBox getSource() {
		return textBox;
	}
	public KriTextBox() {
		super();
		Style styleTextBox = textBox.getElement().getStyle();
		styleTextBox.setBorderWidth(0, Unit.PX);
		styleTextBox.setBackgroundColor("WHITE");
		
		HorizontalPanel horizontalPanel = new HorizontalPanel();
		horizontalPanel.add(textBox);
		horizontalPanel.setCellVerticalAlignment(textBox, HasVerticalAlignment.ALIGN_MIDDLE);
		
		Style styleImage = textBox.getElement().getStyle();
		styleImage.setVerticalAlign(VerticalAlign.MIDDLE);
		
		add(horizontalPanel);
		setCellVerticalAlignment(horizontalPanel, HasVerticalAlignment.ALIGN_MIDDLE);
		
		CSSStyle.maskedTextBox(this);
		
		setBorderWidth(0);
		setSpacing(0);
	}

	public KriTextBox(String regExp, String errorMessage, KriShowMessage errorMessageOn, String helpMessage, KriHelpMessage helpMessageOn) {
		this(regExp, errorMessage, errorMessageOn, helpMessage, helpMessageOn, 150, 28);
	}

	public KriTextBox(String regExp, String errorMessage, KriShowMessage errorMessageOn, final String helpMessage, final KriHelpMessage helpMessageOn, int width, int height) {
		this(regExp, errorMessage, errorMessageOn, width, height);
		this.addFocusHandler(new FocusHandler() {
			@Override
			public void onFocus(FocusEvent event) {
				helpMessageOn.setSuccess(helpMessage);
			}
		});
	}
	
	public KriTextBox(String regExp, String errorMessage, KriShowMessage errorMessageOn, int width, int height) {

		super();
		this.regExp = regExp;
		this.errorMessageOn = errorMessageOn;
		this.errorMessage = errorMessage;
		Style styleTextBox = textBox.getElement().getStyle();
		styleTextBox.setBorderWidth(0, Unit.PX);
		styleTextBox.setBackgroundColor("WHITE");
		
		textBox.addKeyUpHandler(this);
		HorizontalPanel horizontalPanel = new HorizontalPanel();
		horizontalPanel.add(textBox);
		textBox.setWidth((width-10)+"px");
		horizontalPanel.setCellWidth(textBox, width+"px");
		horizontalPanel.setCellVerticalAlignment(textBox, HasVerticalAlignment.ALIGN_MIDDLE);
		horizontalPanel.add(image);
		
		image.setVisible(false);
		Style styleImage = textBox.getElement().getStyle();
		styleImage.setVerticalAlign(VerticalAlign.MIDDLE);
		horizontalPanel.setCellWidth(image, "20px");
		
		horizontalPanel.setCellVerticalAlignment(image, HasVerticalAlignment.ALIGN_MIDDLE);
		
		add(horizontalPanel);
		setCellVerticalAlignment(horizontalPanel, HasVerticalAlignment.ALIGN_MIDDLE);
		
		CSSStyle.maskedTextBox(this);
		
		setBorderWidth(0);
		setSpacing(0);
		setHeight(height+"px");
	}
	public KriTextBox(String regExp, String errorMessage, KriShowMessage errorMessageOn) {
		this(regExp, errorMessage, errorMessageOn, 150, 28);
	}
	
	public String getText() {
		return textBox.getText();
	}

	public TextBox getTextBox() {
		return textBox;
	}

	public void setText(String str) {
		textBox.setText(str);
		image.setVisible(false);
	}
	
	
	public int validate(int validateAllField) {
		
		String str = (""+this.getText()).trim();
		if(str.matches(regExp)) {
			image.setUrl("images/correct16.png");
			errorMessageOn.setError("&nbsp;");
			return validateAllField;
		}else{
			errorMessageOn.setError(errorMessage);
			image.setUrl("images/wrong16.png");
			return ++validateAllField;
		}	
	}
	@Override
	public void onKeyUp(KeyUpEvent event) {
		String str = (""+this.getText()).trim();
		if(str.length() == 0) {
			image.setVisible(false);
			errorMessageOn.setError("&nbsp;");
		}else{
			validate(0);
			image.setVisible(true);
		}
	}
	
	@Override
	public boolean isValidate() {
		return (validate(0)==0);
	}
	@Override
	public void reset() {
		textBox.setText("");
		image.setVisible(false);
	}

	@Override
	public String getValue() {
		return textBox.getText();
	}
}
